/*
 * Copyright (c) 2003-2021 JADAPTIVE Limited. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code and
 * its use is subject to the terms of the JADAPTIVE Software License. You may not use
 * this file except in compliance with the license terms.
 *
 * https://www.jadaptive.com/en/content/api-software-license
 *
 * Email: support@jadaptive.com
 * WWW: https://www.jadaptive.com
 */
import java.io.IOException;
import java.io.InputStream;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import com.maverick.pool.SshClientFactory;
import com.maverick.pool.SshClientPool;
import com.maverick.pool.SshTransferListener;
import com.maverick.pool.SshTransferQueue;
import com.maverick.ssh.ChannelOpenException;
import com.maverick.ssh.PasswordAuthentication;
import com.maverick.ssh.SshClient;
import com.maverick.ssh.SshConnector;
import com.maverick.ssh.SshException;
import com.maverick.ssh.SshSession;
import com.maverick.ssh.SshTransport;
import com.maverick.ssh2.Ssh2Context;
import com.sshtools.net.SocketTransport;


public class ClientPool {

	public static void main(String[] args){

	
		ExecutorService executor = new ThreadPoolExecutor(1000, 1000, 60000,
				TimeUnit.MILLISECONDS, new LinkedBlockingQueue<Runnable>());

		SshClientPool pool = new SshClientPool(executor, new SshClientFactory() {

			public void configureConnector(SshConnector con) throws SshException {
				Ssh2Context ssh2 = (Ssh2Context) con.getContext(SshConnector.SSH2);
				ssh2.setIdleConnectionTimeoutSeconds(120);
			}

			public SshTransport createTransport(String host, int port) throws IOException {
				return new SocketTransport(host, port);
			}

			public void authenticateClient(SshClient client) throws SshException {
				PasswordAuthentication pwd =new PasswordAuthentication();
				pwd.setPassword("xxxxxx");
				client.authenticate(pwd);
			}
			
		});

		try {
			SshClient client1 = pool.checkout("localhost", 22, "lee");
			SshClient client2 = pool.checkout("localhost", 22, "lee");
		
			pool.checkin(client1);
			pool.checkin(client2);
		
			for(int i=0;i<10;i++) {
				
				System.out.println("There are " + pool.getClientCount() + " clients in the pool");
				
				SshClient pooled = pool.checkout("localhost", 22, "lee");
			
				try {
					
					SshSession session = pooled.openSessionChannel();
					session.executeCommand("ls -l");
					
					InputStream in = session.getInputStream();
					byte[] buf = new byte[1024];
					int r;
					while((r = in.read(buf)) > -1) {
						System.out.write(buf, 0, r);
					}
					System.out.flush();
					
					session.close();
					pool.checkin(pooled);
				} catch (ChannelOpenException e) {
					System.out.println("Error opening channel, we will disconnect client and not return it to the pool");
					pooled.disconnect();
				} 
				
			
			}
		
			SshTransferQueue queue = new SshTransferQueue(pool);
			
			queue.postGet("localhost", 22, "lee", "Downloads/test.iso", new SshTransferListener() {
				
				public void transferFailed() {
					System.out.println("Transfer failed");
				}
				
				public void transferComplete() {
					System.out.println("Transfer complete");
					
				}
			});
			
			
			// Now wait, the pooled clients will all disconnect in around 120 seconds.
		
		} catch (SshException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} 
		
		
		
	}
}
