/*
 * Copyright (c) 2003-2021 JADAPTIVE Limited. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code and
 * its use is subject to the terms of the JADAPTIVE Software License. You may not use
 * this file except in compliance with the license terms.
 *
 * https://www.jadaptive.com/en/content/api-software-license
 *
 * Email: support@jadaptive.com
 * WWW: https://www.jadaptive.com
 */

import java.io.BufferedReader;
import java.io.InputStreamReader;

import org.apache.log4j.PropertyConfigurator;

import com.maverick.ssh.Shell;
import com.maverick.ssh.ShellProcess;
import com.maverick.ssh.SshAuthentication;
import com.maverick.ssh.SshClient;
import com.maverick.ssh.SshClientConnector;
import com.maverick.ssh.SshConnector;
import com.maverick.ssh1.Ssh1Client;
import com.sshtools.net.SocketTransport;
/**
 * This example demonstrates the latest feature addition to the API. The use
 * of an expect type session to control a remote process.
 *
 * @author Lee David Painter
 */
public class ExpectShell {

	public static void main(String[] args) {

		final BufferedReader reader = new BufferedReader(new InputStreamReader(
				System.in));

		PropertyConfigurator.configure("log4j.properties");
		
		try {
			System.out.print("Hostname: ");
			String hostname;
			hostname = reader.readLine();

			int idx = hostname.indexOf(':');
			int port = 22;
			if (idx > -1) {
				port = Integer.parseInt(hostname.substring(idx + 1));
				hostname = hostname.substring(0, idx);

			}

			System.out.print("Username [Enter for "
					+ System.getProperty("user.name") + "]: ");
			String username;
			username = reader.readLine();

			if (username == null || username.trim().equals(""))
				username = System.getProperty("user.name");

			System.out.println("Connecting to " + hostname);

			/**
			 * Create an SshConnector instance
			 */
			SshClientConnector con = SshConnector.createInstance();

			/**
			 * Connect to the host
			 */
			final SshClient ssh = con.connect(new SocketTransport(hostname,
					port), username);

			/**
			 * Determine the version
			 */
			if (ssh instanceof Ssh1Client)
				System.out.println(hostname + " is an SSH1 server");
			else
				System.out.println(hostname + " is an SSH2 server");

			/**
			 * Authenticate the user using password authentication
			 */
			com.maverick.ssh.PasswordAuthentication pwd = new com.maverick.ssh.PasswordAuthentication();

			do {
				System.out.print("Password: ");
				pwd.setPassword(reader.readLine());
			} while (ssh.authenticate(pwd) != SshAuthentication.COMPLETE
					&& ssh.isConnected());

			/**
			 * Start a session and do basic IO
			 */
			if (ssh.isAuthenticated()) {

				/**
				 * Create a Shell
				 */
				Shell shell = new Shell(ssh);
				
				/**
				 * Clear the sudo timestamp so that a password is requested
				 */
				shell.executeCommand("sudo -k", false, true);
				
				/**
				 * Execute a command under the new shell, enter the password when prompted
				 * NOTE: this requires a password prompt to use sudo -k to ensure it does
				 */
				Shell rootShell= shell.su("sudo -iu root", "xxxxxx");
				
				ShellProcess process = rootShell.executeCommand("whoami");
				
				int r;
				while((r = process.getInputStream().read()) > -1) {
					System.out.write((char)r);
					System.out.flush();
				}
				
			
				rootShell.exit();
				
				
				process = shell.sudo("sudo touch file", "xxxxxx");
				process.drain();
				
				/** 
				 * Exit
				 */
				shell.exit();
			}

			ssh.disconnect();
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

}

