/*
 * Copyright (c) 2003-2021 JADAPTIVE Limited. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code and
 * its use is subject to the terms of the JADAPTIVE Software License. You may not use
 * this file except in compliance with the license terms.
 *
 * https://www.jadaptive.com/en/content/api-software-license
 *
 * Email: support@jadaptive.com
 * WWW: https://www.jadaptive.com
 */

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;

import com.maverick.agent.client.AgentForwardingChannelFactory;
import com.maverick.agent.client.AgentSocketType;
import com.maverick.agent.client.Ssh2AgentAuthentication;
import com.maverick.agent.client.SshAgentClient;
import com.maverick.ssh.HostKeyVerification;
import com.maverick.ssh.SshAuthentication;
import com.maverick.ssh.SshConnector;
import com.maverick.ssh.SshException;
import com.maverick.ssh.SshSession;
import com.maverick.ssh.components.SshPublicKey;
import com.maverick.ssh2.Ssh2Client;
import com.maverick.ssh2.Ssh2Session;
import com.sshtools.net.SocketTransport;
/**
 * This example demonstrates the connection process using the standard SSH interfaces
 * that enable connections to both SSH1 and SSH2 servers and using public key authentication.
 *
 * @author Lee David Painter
 */
public class OpenSSHAgentKeyConnect {

	public static void main(String[] args) throws Exception {

		final BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));

		//PropertyConfigurator.configure("log4j.properties");
		
		final String SSH_AUTH_SOCK = System.getenv("SSH_AUTH_SOCK");		
		SshAgentClient agentClient = SshAgentClient.connectLocalAgent("Test", 
				SSH_AUTH_SOCK, AgentSocketType.UNIX_DOMAIN, false);
		
		try {
			
			System.out.print("Hostname: ");
			String hostname = reader.readLine();

			int idx = hostname.indexOf(':');
			int port = 22;
			if(idx > -1) {
				port = Integer.parseInt(hostname.substring(idx+1));
				hostname = hostname.substring(0, idx);

			}

			System.out.print("Username [Enter for " + System.getProperty("user.name") + "]: ");
			String username = reader.readLine();

			if(username==null || username.trim().equals(""))
				username = System.getProperty("user.name");

			System.out.println("Connecting to " + hostname);

			/**
			 * Create an SshConnector instance
			 */
			SshConnector con = SshConnector.createInstance();

                        //con.setSupportedVersions(1);
			// Lets do some host key verification
			HostKeyVerification hkv = new HostKeyVerification() {
			public boolean verifyHost(String hostname, SshPublicKey key) {
				try {
					System.out.println("The connected host's key (" + key.getAlgorithm() + ") is");
					System.out.println(key.getFingerprint());
				} catch (SshException e) {
				}
				return true;
			}
			};

			con.setKnownHosts(hkv);

	
			/**
			 * Connect to the host
			 */
			Ssh2Client ssh = (Ssh2Client) con.connect(new SocketTransport(hostname, port), username, true);

			ssh.addChannelFactory(new AgentForwardingChannelFactory(SSH_AUTH_SOCK, AgentSocketType.UNIX_DOMAIN));

			/**
			 * Authenticate the user using password authentication
			 */
			Ssh2AgentAuthentication pk = new Ssh2AgentAuthentication(agentClient);
			
			if(ssh.authenticate(pk)!=SshAuthentication.COMPLETE) {
				throw new Exception("Agent authentication failed");
			}
			
			/**
			 * Start a session and do basic IO
			 */
			if(ssh.isAuthenticated()) {

				SshSession session = ssh.openSessionChannel();
				session.requestPseudoTerminal("vt100",80,24,0,0);
				
				Ssh2Session ssh2Session = (Ssh2Session) session;
				if(!ssh2Session.sendRequest("auth-agent-req@openssh.com", true, null)) {
					System.out.println("Agent request failed");
				}
				
				session.startShell();

			
				
				final InputStream in = session.getInputStream();
				Thread t = new Thread(new Runnable() {
					public void run() {
						try {
							int read;
							while((read = in.read()) > -1) {
								if(read > 0)
									System.out.print((char)read);
							}
						} catch(Throwable t1) {
							t1.printStackTrace();
						} finally {
							System.exit(0);
						}
					}

				});
				t.start();
				int read;
				while((read = System.in.read()) > -1) {
					session.getOutputStream().write(read);
				}

				System.exit(0);
			}


			} catch(Throwable th) {
			th.printStackTrace();
		} finally {
			agentClient.close();
		}
	}

}



