/*
 * Copyright (c) 2003-2021 JADAPTIVE Limited. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code and
 * its use is subject to the terms of the JADAPTIVE Software License. You may not use
 * this file except in compliance with the license terms.
 *
 * https://www.jadaptive.com/en/content/api-software-license
 *
 * Email: support@jadaptive.com
 * WWW: https://www.jadaptive.com
 */

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import org.apache.log4j.PropertyConfigurator;

import com.maverick.ssh.PseudoTerminalModes;
import com.maverick.ssh.SshAuthentication;
import com.maverick.ssh.SshClient;
import com.maverick.ssh.SshClientAdapter;
import com.maverick.ssh.SshConnector;
import com.maverick.ssh.SshException;
import com.maverick.ssh.SshKeyFingerprint;
import com.maverick.ssh.SshSession;
import com.maverick.ssh.components.SshPublicKey;
import com.maverick.ssh1.Ssh1Client;
import com.maverick.ssh2.Ssh2Context;
import com.sshtools.net.SocketTransport;
import com.sshtools.publickey.ConsoleKnownHostsFile;
/**
 * This example demonstrates the connection process using the standard SSH interfaces
 * that enable connections to both SSH1 and SSH2 servers.
 *
 * @author Lee David Painter
 */
public class PasswordConnect {

	public static void main(String[] args) {

		final BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));

		PropertyConfigurator.configure("log4j.properties");
		
		try {
			System.out.print("Hostname: ");
            String hostname;
            hostname = reader.readLine();

            int idx = hostname.indexOf(':');
            int port = 22;
            if (idx > -1) {
                port = Integer.parseInt(hostname.substring(idx + 1));
                hostname = hostname.substring(0, idx);

            }

            System.out.print("Username [Enter for " + System.getProperty("user.name") +
                             "]: ");
            String username;
            username = reader.readLine();

            if (username == null || username.trim().equals(""))
                username = System.getProperty("user.name");

            
            
            
            /**
             * Create an SshConnector instance
             */
            SshConnector con = SshConnector.createInstance();
            
            // Verify server host keys using the users known_hosts file
            con.setKnownHosts(new ConsoleKnownHostsFile());
            Ssh2Context ctx = (Ssh2Context) con.getContext(2);
            ctx.setPreferredCipherSC("aes128-ctr");
            ctx.setPreferredCipherCS("aes128-ctr");
            ctx.setPreferredMacSC("hmac-sha2-512-etm@openssh.com");
            ctx.setPreferredMacCS("hmac-sha2-512-etm@openssh.com");
            
            /**
             * Connect to the host
             */
            ((Ssh2Context)con.getContext(SshConnector.SSH2)).setSendIgnorePacketOnIdle(false);
            
            System.out.println("Connecting to " + hostname);
            
            SocketTransport transport = new SocketTransport(hostname, port);
            
            System.out.println("Creating SSH client");
            
            con.addListener(new SshClientAdapter() {

				@Override
				public void keyExchangeComplete(SshClient client, SshPublicKey hostkey, String keyExchange,
						String cipherCS, String cipherSC, String macCS, String macSC, String compressionCS,
						String compressionSC) {
					try {
						System.out.println("Got host key "  + SshKeyFingerprint.getFingerprint(hostkey.getEncoded()));
					} catch (SshException e) {
					}
				}
            	
            });
            final SshClient ssh = con.connect(transport,
                                              username, true);
            
            /**
             * Determine the version
             */
            if (ssh instanceof Ssh1Client)
                System.out.println(hostname + " is an SSH1 server");
            else
                System.out.println(hostname + " is an SSH2 server");

            /**
             * Authenticate the user using password authentication
             */
            com.maverick.ssh.PasswordAuthentication pwd = new com.maverick.ssh.
                                                          PasswordAuthentication();

            do {
                System.out.print("Password: ");
                pwd.setPassword(reader.readLine());
            } while (ssh.authenticate(pwd) != SshAuthentication.COMPLETE
                     && ssh.isConnected());

            /**
             * Start a session and do basic IO
             */
            if (ssh.isAuthenticated()) {

                final SshSession session = ssh.openSessionChannel();

                // Use the newly added PseudoTerminalModes class to
                // turn off echo on the remote shell
                PseudoTerminalModes pty = new PseudoTerminalModes(ssh);
                pty.setTerminalMode(PseudoTerminalModes.ECHO, false);

                session.requestPseudoTerminal("vt100", 80, 24, 0, 0, pty);

                session.startShell();

                Thread t = new Thread() {
                    public void run() {
                        try {
                        	int read;
                            while((read = System.in.read()) > -1) {
                                session.getOutputStream().write(read);
                            }
                        } catch (IOException ex) {
                            ex.printStackTrace();
                        }
                    }
                };

                t.start();
                
                int read;
                while ((read = session.getInputStream().read()) > -1) {
                    System.out.write(read);
                    System.out.flush();
                }

                session.close();
            }

            ssh.disconnect();
        } catch(Throwable t) {
            t.printStackTrace();
        }
	}

}

