/*
 * Copyright (c) 2003-2021 JADAPTIVE Limited. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code and
 * its use is subject to the terms of the JADAPTIVE Software License. You may not use
 * this file except in compliance with the license terms.
 *
 * https://www.jadaptive.com/en/content/api-software-license
 *
 * Email: support@jadaptive.com
 * WWW: https://www.jadaptive.com
 */
import java.io.IOException;

import com.maverick.ssh.SecureComponent;
import com.maverick.ssh.SecurityLevel;
import com.maverick.ssh.SshException;
import com.maverick.ssh.components.ComponentManager;
import com.maverick.ssh.components.jce.JCEComponentManager;

public class SecurityReporter {

	public static void main(String[] args) throws SshException, IOException {
		
		
		ComponentManager jce = JCEComponentManager.getInstance();
		
		jce.setMinimumSecurityLevel(SecurityLevel.WEAK);
		
		
		System.out.println(String.format("%-15s%-50s%s", "Security Level", "Algorithm", "Score"));
		System.out.println("#####################################################################################");
		System.out.println("Ciphers");
		System.out.println("-------");
		for(String name : jce.supportedSsh2CiphersCS().order()) {
			SecureComponent obj = (SecureComponent) jce.supportedSsh2CiphersCS().getInstance(name);
			System.out.println(String.format("%-15s%-50s%d", obj.getSecurityLevel().name(), obj.getAlgorithm(), obj.getPriority()));
		}
		
		System.out.println();
		System.out.println("Macs");
		System.out.println("----");
		for(String name : jce.supportedHMacsCS().order()) {
			SecureComponent obj = (SecureComponent) jce.supportedHMacsCS().getInstance(name);
			System.out.println(String.format("%-15s%-50s%d", obj.getSecurityLevel().name(), obj.getAlgorithm(), obj.getPriority()));
		}
		
		System.out.println();
		System.out.println("Public Keys");
		System.out.println("-----------");
		for(String name : jce.supportedPublicKeys().order()) {
			SecureComponent obj = (SecureComponent) jce.supportedPublicKeys().getInstance(name);
			System.out.println(String.format("%-15s%-50s%d", obj.getSecurityLevel().name(), obj.getAlgorithm(), obj.getPriority()));
		}
		
		System.out.println();
		System.out.println("Key Exchange");
		System.out.println("------------");
		for(String name : jce.supportedKeyExchanges(false).order()) {
			SecureComponent obj = (SecureComponent) jce.supportedKeyExchanges(false).getInstance(name);
			System.out.println(String.format("%-15s%-50s%d", obj.getSecurityLevel().name(), obj.getAlgorithm(), obj.getPriority()));
		}

	}

}
