/*
 * Copyright (c) 2003-2021 JADAPTIVE Limited. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code and
 * its use is subject to the terms of the JADAPTIVE Software License. You may not use
 * this file except in compliance with the license terms.
 *
 * https://www.jadaptive.com/en/content/api-software-license
 *
 * Email: support@jadaptive.com
 * WWW: https://www.jadaptive.com
 */
import java.io.IOException;
import java.util.List;
import java.util.Set;

import javax.swing.JOptionPane;

import com.maverick.ssh.SshException;
import com.maverick.ssh.components.SshPublicKey;
import com.maverick.util.SshKeyUtils;
import com.sshtools.publickey.KnownHostsFile;

public class SwingKnownHostsPrompt extends KnownHostsFile {

	public SwingKnownHostsPrompt() throws SshException, IOException {
		super();
	}

	private String getKeyFingerprints(List<SshPublicKey> keys) throws SshException {
		StringBuffer buf = new StringBuffer();
		for(SshPublicKey key : keys) {
			if(buf.length() > 0) {
				buf.append("\r\n");
			}
			buf.append(key.getAlgorithm() + " " + key.getFingerprint());
		}
		return buf.toString();
	}
	
	@Override
	protected void onHostKeyMismatch(String host, List<SshPublicKey> allowedHostKey, SshPublicKey actualHostKey)
			throws SshException {
		int response = JOptionPane.showConfirmDialog(null, 
				String.format("%s is the known key for %s\r\nThe server has presented a different key %s\r\n\r\nDo you want to accept the key?", 
						getKeyFingerprints(allowedHostKey),
						host,
						actualHostKey.getFingerprint()),
						"Host Key Mismatch",
						JOptionPane.YES_NO_OPTION);
		if(response == JOptionPane.YES_OPTION) {
			addEntry(actualHostKey, "Added by " + System.getProperty("user.name"), host);
		}
	}

	
	@Override
	protected void onInvalidHostEntry(String entry) throws SshException {
		JOptionPane.showMessageDialog(null, 
				"There is an invalid entry in the known_hosts file!\r\n\r\n"
				+ entry);
	}
	
	@Override
	protected void onRevokedKey(String host, SshPublicKey key) {
		JOptionPane.showMessageDialog(null, 
				"The key has been marked as revoked!\r\n\r\n"
				+ key.getAlgorithm() + " " + SshKeyUtils.getFingerprint(key));
	}

	@Override
	protected void onHostKeyUpdated(Set<String> names, SshPublicKey key) {
		
	}


	@Override
	public void onUnknownHost(String host, SshPublicKey key) throws SshException {
		
		int response = JOptionPane.showConfirmDialog(null, 
				String.format("%s is not a known host key for %s\r\n\r\nDo you want to accept the key?", 
						key.getAlgorithm() + " " + key.getFingerprint(),
						host),
						"Unknown Host",
						JOptionPane.YES_NO_OPTION);
		if(response == JOptionPane.YES_OPTION) {
			addEntry(key, "Added by " + System.getProperty("user.name"), host);
		}
	}

}
